/* ============================================================
 *
 * This file is a part of digiKam project
 * http://www.digikam.org
 *
 * Date        : 2005-03-11
 * Description : a digiKam image editor plugin to correct 
 *               image white balance 
 * 
 * Copyright (C) 2005-2008 by Gilles Caulier <caulier dot gilles at gmail dot com>
 * Copyright (C) 2008 by Guillaume Castagnino <casta at xwing dot info>
 * 
 * This program is free software; you can redistribute it
 * and/or modify it under the terms of the GNU General
 * Public License as published by the Free Software Foundation;
 * either version 2, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * ============================================================ */

#ifndef IMAGEEFFECT_WHITEBALANCE_H
#define IMAGEEFFECT_WHITEBALANCE_H

// TQt include.

#include <tqcolor.h>

// Digikam includes.

#include "imagedlgbase.h"

class TQPushButton;
class TQLabel;
class TQComboBox;
class TQPushButton;
class TQHButtonGroup;

class KDoubleNumInput;
class KActiveLabel;

namespace Digikam
{
class HistogramWidget;
class ColorGradientWidget;
class ImageWidget;
class DColor;
}

namespace DigikamWhiteBalanceImagesPlugin
{

class ImageEffect_WhiteBalance : public Digikam::ImageDlgBase
{
    TQ_OBJECT
  

public:

    ImageEffect_WhiteBalance(TQWidget* parent);
    ~ImageEffect_WhiteBalance();

protected:

    void finalRendering();    

private slots:

    void slotUser2();
    void slotUser3();
    void slotEffect();
    void slotColorSelectedFromOriginal(const Digikam::DColor &color);
    void slotColorSelectedFromTarget(const Digikam::DColor &color);
    void slotScaleChanged(int scale);
    void slotChannelChanged(int channel);
    void slotTemperatureChanged(double temperature);
    void slotTemperaturePresetChanged(int tempPreset);
    void slotAutoAdjustExposure(void);
    void slotPickerColorButtonActived();    

private:

    void readUserSettings();
    void writeUserSettings();
    void resetValues();

private:    

    enum HistogramScale
    {
        Linear=0,
        Logarithmic
    };
    
    enum ColorChannel
    {
        LuminosityChannel=0,
        RedChannel,
        GreenChannel,
        BlueChannel
    };
    
    enum TemperaturePreset
    {
        Candle=0,
        Lamp40W,
        Lamp100W,
        Lamp200W,
        Sunrise,
        StudioLamp,
        MoonLight,
        Neutral,
        DaylightD50,
        Flash,
        Sun,
        XeonLamp,
        DaylightD65,
        None
    };

    uchar                        *m_destinationPreviewData;

    int                           m_currentPreviewMode;

    TQPushButton                  *m_pickTemperature;
    TQPushButton                  *m_autoAdjustExposure;
    
    TQComboBox                    *m_temperaturePresetCB;    
    TQComboBox                    *m_channelCB;    
    
    TQHButtonGroup                *m_scaleBG;  
        
    TQLabel                       *m_adjTemperatureLabel;
    TQLabel                       *m_temperaturePresetLabel;
    TQLabel                       *m_darkLabel;
    TQLabel                       *m_blackLabel;
    TQLabel                       *m_mainExposureLabel;
    TQLabel                       *m_fineExposureLabel;
    TQLabel                       *m_gammaLabel;
    TQLabel                       *m_saturationLabel;
    TQLabel                       *m_greenLabel;
    
    KActiveLabel                 *m_exposureLabel;
    KActiveLabel                 *m_temperatureLabel;

    KDoubleNumInput              *m_temperatureInput;
    KDoubleNumInput              *m_darkInput;
    KDoubleNumInput              *m_blackInput;
    KDoubleNumInput              *m_mainExposureInput;
    KDoubleNumInput              *m_fineExposureInput;
    KDoubleNumInput              *m_gammaInput;
    KDoubleNumInput              *m_saturationInput;
    KDoubleNumInput              *m_greenInput;
    
    Digikam::HistogramWidget     *m_histogramWidget;
    
    Digikam::ColorGradientWidget *m_hGradient;
    
    Digikam::ImageWidget         *m_previewWidget;
};

}  // NameSpace DigikamWhiteBalanceImagesPlugin

#endif /* IMAGEEFFECT_WHITEBALANCE_H */
